@@ -6,9 +6,10 @@ import subprocess
 import acme_tiny
 
 
-def generate_cert(domain, cert_dir):
+def generate_cert(domain, cert_dir="certs"):
     """Generate a TLS certificate signed by Let's Encrypt for given domain."""
-    cert_dir = pathlib.Path(cert_dir)
+    cert_dir = pathlib.Path(cert_dir) / domain
+    cert_dir.mkdir(exist_ok=True)
     account_key = cert_dir / "account.key"
     if not account_key.exists():
         with account_key.open("w") as fp:
@@ -29,10 +30,14 @@ def generate_cert(domain, cert_dir):
                     "-key",
                     private_key,
                     "-subj",
-                    f"/CN={domain}",
+                    '"/"',
+                    "-addext",
+                    f'"subjectAltName = DNS:{domain}, DNS:www.{domain}"',
                 ],
                 stdout=fp,
             )
+    challenge_dir = cert_dir / "challenges"
+    challenge_dir.mkdir(exist_ok=True)
     with (cert_dir / "domain.crt").open("w") as fp:
-        fp.write(acme_tiny.get_crt(account_key, csr, cert_dir / "challenges"))
-    csr.unlink()
+        fp.write(acme_tiny.get_crt(account_key, csr, challenge_dir))
+    # XXX csr.unlink()
