@@ -330,6 +330,9 @@ class Machine(MachineBase):
         "htop",
         "stow",  # for dotfiles
         "zsh",  # default shell
+        "libc6-dev",  # onion generation
+        "libsodium-dev",  # ""
+        "autoconf",  # ""
     )
     versions = {
         "python": "3.10.9",
@@ -409,11 +412,23 @@ class Machine(MachineBase):
 
         """
         self._apt("install -yq nginx libnginx-mod-rtmp")
-        self.nginx_conf = "/etc/nginx/nginx.conf"
+        self.nginx_conf = "/etc/nginx"
         self.open_ports(80, 443, 1935)
         self.run(
-            f"cat > {self.nginx_conf}",
-            stdin=str(templates.nginx_conf(self.address, self.onion)),
+            f"cat > {self.nginx_conf}/acme-challenge.conf",
+            stdin=str(templates.nginx_acme_challenge(self.run_dir)),
+        )
+        self.run(
+            f"cat > {self.nginx_conf}/ssl-params.conf",
+            stdin=str(templates.nginx_ssl_params(f"{self.nginx_conf}/dhparam.pem")),
+        )
+        self.run(
+            f"cat > {self.nginx_conf}/application.conf",
+            stdin=str(templates.nginx_application(self.run_dir, self.onion)),
+        )
+        self.run(
+            f"cat > {self.nginx_conf}/nginx.conf",
+            stdin=str(templates.nginx(self.address, [self.onion], {})),
         )
 
         self.user = "gaea"
@@ -425,6 +440,7 @@ class Machine(MachineBase):
         self.run(f"mkdir {streaming_dir}/rec")
 
         # self-signed TLS certificate for IP address
+        self.run(f"mkdir {self.run_dir}/certs -p")
         domain_cnf = configparser.ConfigParser()
         domain_cnf.optionxform = str
         domain_cnf["req"] = {
@@ -441,14 +457,14 @@ class Machine(MachineBase):
         config_output = io.StringIO()
         domain_cnf.write(config_output)
         self.run(
-            f"cat > /home/gaea/app/run/ip-selfsigned.cnf",
+            f"cat > {self.run_dir}/certs/selfsigned-ip.cnf",
             stdin=config_output.getvalue(),
         )
         self.run(
             "sudo openssl req -x509 -nodes -days 365 -newkey rsa:2048"
-            " -keyout /etc/ssl/private/ip-selfsigned.key"
-            " -out /etc/ssl/certs/ip-selfsigned.crt"
-            " -config /home/gaea/app/run/ip-selfsigned.cnf"
+            " -keyout /etc/ssl/private/selfsigned-ip.key"
+            " -out /etc/ssl/certs/selfsigned-ip.crt"
+            f" -config {self.run_dir}/certs/selfsigned-ip.cnf"
         )
 
         self.user = "root"
@@ -472,14 +488,19 @@ class Machine(MachineBase):
 
     def generate_dhparam(self, bits=4096):
         """Generate a unique Diffie-Hellman prime for Nginx."""
-        self.run(f"openssl dhparam -out /etc/nginx/dhparam.pem {bits}")
+        self.run(f"openssl dhparam -out {self.nginx_conf}/dhparam.pem {bits}")
         self.run("service nginx restart")
 
-    def setup_app(self, package, app):
+    def setup_app(self, package, app, do_token=None):
         """Install administrative interface."""
         self.user = "gaea"
         self.run("python3 -m venv app")
         self.run(f"mkdir {self.run_dir}")
+        if do_token:
+            self.run(
+                f"cat > {self.run_dir}/webcfg.ini",
+                stdin=f'DIGITALOCEAN_TOKEN = "{do_token}"',
+            )
         self.run(f"{self.runinenv} app pip install {package}")
         self.user = "root"
         with self.supervisor(package) as config:
